#!/usr/bin/env python

from __future__ import print_function

import argparse
import os
import sys
import tarfile
import tempfile
import shutil
import subprocess
import distutils.dir_util

def parse_cli():
    arg_parser = argparse.ArgumentParser()
    arg_parser.add_argument('package', help='Path to the Vuforia Engine Unity Extension package, i.e. *.tgz or *.unitypackage.')
    return arg_parser.parse_args()

def replace_signature(base_path):
    bundle_path = os.path.join(base_path, 'package/Vuforia/Plugins/OSXUniversal/universal/VuforiaEngine.bundle')

    tmp_bundle = os.path.join(tempfile.gettempdir(), 'VuforiaEngine.bundle')
    if os.path.exists(tmp_bundle):
        shutil.rmtree(tmp_bundle)
    shutil.copytree(bundle_path, tmp_bundle, ignore=shutil.ignore_patterns('*.meta'))

    with open(os.devnull, 'wb') as silence:
        try:
            subprocess.check_call(['codesign', '--force', '--deep', '--sign', '-', tmp_bundle], stdout=silence, stderr=silence)
        except subprocess.CalledProcessError:
            subprocess.call(['xcode-select', '--install'], stdout=silence, stderr=silence)
            sys.exit(
                "Replacing bundle signature failed.\n"
                "Xcode Command Line Tools are required, requesting installation. "
                "Please re-run this script once the Xcode Command Line Tools have been installed.")

    distutils.dir_util.copy_tree(tmp_bundle, bundle_path)
    shutil.rmtree(tmp_bundle)

def create_tmp_workspace(name):
    tmp_workspace = os.path.join(tempfile.gettempdir(), name)

    if os.path.exists(tmp_workspace):
        shutil.rmtree(tmp_workspace)

    os.mkdir(tmp_workspace)

    return tmp_workspace

def backup_package(package):
    package_bkp = package + '_bkp'
    if not os.path.exists(package_bkp):
        shutil.copy(package, package_bkp)

def patch_vuforia_tgz(package):
    print('Patching Vuforia Engine package - .tgz')

    print('  * Extracting Vuforia Engine package')
    tmp_workspace = create_tmp_workspace(os.path.basename(package))
    with tarfile.open(package, 'r:gz') as package_archive:
        package_archive.extractall(tmp_workspace)

    print('  * Replacing bundle signature')
    replace_signature(tmp_workspace)

    print('  * Repacking Vuforia Engine package')
    os.remove(package)

    with tarfile.open(package, 'w:gz') as new_package_archive:
        new_package_archive.add(tmp_workspace, arcname='')

    shutil.rmtree(tmp_workspace)

def patch_vuforia_unitypackage(package):
    print('Patching Vuforia Engine package - .unitypackage')

    tmp_workspace = create_tmp_workspace(os.path.basename(package))

    print('  * Extracting Vuforia Engine package')
    with tarfile.open(package, 'r:gz') as package_archive:
        sorted_tar_files = sorted(package_archive.getmembers(), key=lambda m: m.size, reverse=True)
        main_assets = os.path.join(tmp_workspace, sorted_tar_files[0].name)
        main_assests_dir = os.path.dirname(main_assets)

        package_archive.extractall(tmp_workspace)

    with tarfile.open(main_assets, 'r:gz') as main_asset_tar:
        main_asset_tar.extractall(main_assests_dir)

    print('  * Replacing bundle signature')
    replace_signature(main_assests_dir)

    os.remove(main_assets)

    print('  * Repacking Vuforia Engine package')
    with tarfile.open(main_assets, 'w:gz') as new_main_assets_tar:
        new_main_assets_tar.add(os.path.join(main_assests_dir, 'package'), arcname='package')

    shutil.rmtree(os.path.join(main_assests_dir, 'package'))

    os.remove(package)

    with tarfile.open(package, 'w:gz') as new_package_archive:
        new_package_archive.add(tmp_workspace, arcname='.')

    shutil.rmtree(tmp_workspace)

def main():
    cli_args = parse_cli()

    if not os.path.exists(cli_args.package):
        sys.exit("Vuforia Engine package file not found: {}".format(cli_args.package))

    backup_package(cli_args.package)

    package_ext = os.path.splitext(cli_args.package)[1]
    if package_ext == '.unitypackage':
        patch_vuforia_unitypackage(cli_args.package)
    elif package_ext == '.tgz':
        patch_vuforia_tgz(cli_args.package)
    else:
        sys.exit('Unknown extension of provided package. Please provide path to the Vuforia Engine package with .tgz or .unitypackage extension.')

if __name__ == "__main__":
    main()
